#include "dto.h"

namespace dto::Responses {
    drogon::HttpResponsePtr get_error_res(drogon::HttpStatusCode code, const std::string& msg) {
        Json::Value json;
        json["statusCode"] = static_cast<int>(code);
        json["message"] = msg;
        auto res = drogon::HttpResponse::newHttpJsonResponse(json);
        res->setStatusCode(code);
        return res;
    }

    drogon::HttpResponsePtr get_success_res() {
        Json::Value json;
        return get_success_res(json);
    }

    drogon::HttpResponsePtr get_success_res(Json::Value& json) {
        json["statusCode"] = 200;
        auto res = drogon::HttpResponse::newHttpJsonResponse(json);
        res->setStatusCode(drogon::HttpStatusCode::k200OK);
        return res;
    }

    drogon::HttpResponsePtr get_login_res(const std::string &jwt) {
        Json::Value json;
        json["jwt"] = jwt;
        return get_success_res(json);
    }

    drogon::HttpResponsePtr get_tfa_setup_res(const std::string& secret, const std::string& qrcode) {
        Json::Value json;
        json["secret"] = secret;
        json["qrCode"] = qrcode;
        return get_success_res(json);
    }

    drogon::HttpResponsePtr get_user_info_res(const std::string &name, bool gitlab, bool tfa) {
        Json::Value json;
        json["name"] = name;
        json["gitlab"] = gitlab;
        json["tfaEnabled"] = tfa;
        return get_success_res(json);
    }

    drogon::HttpResponsePtr get_admin_users_res(const std::vector<GetUsersEntry>& users) {
        Json::Value json;
        for (const GetUsersEntry& user : users) {
            Json::Value entry;
            entry["id"] = user.id;
            entry["gitlab"] = user.gitlab;
            entry["name"] = user.name;
            entry["role"] = user.role;
            entry["tfaEnabled"] = user.tfa;
            json["users"].append(entry);
        }
        return get_success_res(json);
    }

    drogon::HttpResponsePtr get_root_res(uint64_t root) {
        Json::Value json;
        json["rootId"] = root;
        return get_success_res(json);
    }

    drogon::HttpResponsePtr get_node_folder_res(uint64_t id, const std::string &name, const std::shared_ptr<uint64_t> &parent, const std::vector<uint64_t> &children) {
        Json::Value json;
        json["id"] = id;
        json["name"] = name;
        json["isFile"] = false;
        json["parent"] = (parent != nullptr) ? *parent : Json::Value::nullSingleton();
        for (uint64_t child : children)
            json["children"].append(child);
        return get_success_res(json);
    }

    drogon::HttpResponsePtr get_node_file_res(uint64_t id, const std::string &name, const std::shared_ptr<uint64_t> &parent, uint64_t size) {
        Json::Value json;
        json["id"] = id;
        json["name"] = name;
        json["isFile"] = true;
        json["parent"] = (parent != nullptr) ? *parent : Json::Value::nullSingleton();
        json["size"] = size;
        return get_success_res(json);
    }

    drogon::HttpResponsePtr get_path_res(const std::string& path) {
        Json::Value json;
        json["path"] = path;
        return get_success_res(json);
    }

    drogon::HttpResponsePtr get_new_node_res(uint64_t id) {
        Json::Value json;
        json["id"] = id;
        return get_success_res(json);
    }
}
